mod data;
mod redirecter;
mod status;

use std::time::Duration;

use data::Data;
use redirecter::Redirecter;
use rocket::{
    fs::{FileServer, Options},
    get,
    response::content::RawHtml,
    routes, State,
};
use tokio::time::Instant;

#[get("/")]
async fn index(data: &State<Data>) -> RawHtml<String> {
    index_gen(data, false).await
}

#[get("/dbg")]
async fn index_dbg(data: &State<Data>) -> RawHtml<String> {
    index_gen(data, true).await
}

async fn index_gen(data: &State<Data>, dbg: bool) -> RawHtml<String> {
    let mut o = "<!DOCTYPE html><html><head>".to_owned();
    o.push_str(r#"<meta charset="UTF-8">"#);
    o.push_str(r#"<meta name="color-scheme" content="light dark">"#);
    o.push_str(r#"<meta name="viewport" content="width=device-width, initial-scale=1">"#);
    o.push_str("<title>tomatenmhark</title>");
    o.push_str(
        "<style>
a:link {
  color: DarkCyan;
  text-decoration: none;
  font-weight: normal;
}
a:hover {
  color: DarkCyan;
  text-decoration: none;
  font-weight: bold;
}
a:visited {
  color: DarkCyan;
  text-decoration: underline;
  font-weight: normal;
}
span {
  color: SeaGreen;
  font-weight: normal;
}
.ohs {
  display: none;
}
.oht:hover + .ohs, .ohs:hover {
  display: block;
}
</style>
",
    );
    o.push_str("</head><body>");
    o.push_str("<h2>:) tomatenmhark :)</h2>");
    let status = data.status_async(dbg).await;
    let time_queried = Instant::now();
    if !status.0.is_empty() {
        o.push_str("<h3>things running on the server</h3>");
        o.push_str("<ul>");
        for (id, (info, redirect, status, additional, dur)) in status.0.iter() {
            o.push_str("<li>");
            if *info {
                o.push_str(r#"<a href="/info/"#);
                o.push_str(&html_escape::encode_double_quoted_attribute(&id));
                o.push_str(r#"">"#);
            } else if *redirect {
                o.push_str(r#"<a href="/"#);
                o.push_str(&html_escape::encode_double_quoted_attribute(&id));
                o.push_str(r#"">"#);
            } else {
                o.push_str("<span>");
            }
            o.push_str(&html_escape::encode_text(&id));
            if *info || *redirect {
                o.push_str("</a>");
            } else {
                o.push_str("</span>");
            }
            o.push_str(": ");
            if additional.is_some() {
                o.push_str(r#"<em class="oht">"#);
            }
            o.push_str(&html_escape::encode_text(&status));
            if let Some(additional) = additional {
                o.push_str(r#"</em><div class="ohs">"#);
                o.push_str(
                    &html_escape::encode_text(additional)
                        .replace('\r', "")
                        .replace('\n', "<br>"),
                );
                o.push_str("</div>");
            }
            if dbg {
                if let Some(dur) = *dur {
                    o.push_str(&format!(" | {}ms", dur.as_millis()));
                }
            }
            o.push_str("</li>");
        }
        o.push_str("</ul>");
    }
    if !status.1.is_empty() {
        o.push_str("<h3>everything else</h3>");
        o.push_str("<ul>");
        for (id, text) in status.1.iter() {
            o.push_str("<li>");
            o.push_str(r#"<a href="/info/"#);
            o.push_str(&html_escape::encode_double_quoted_attribute(&id));
            o.push_str(r#"">"#);
            o.push_str(&html_escape::encode_text(&id));
            o.push_str("</a>");
            o.push_str(": ");
            o.push_str(&html_escape::encode_text(&text));
            o.push_str("</li>");
        }
        o.push_str("</ul>");
    }
    o.push_str(
        r#"<br><small><small><a href="/int">tomatenmharksite documentation</a></small></small>"#,
    );
    o.push_str(r#"<br><br><br><br><br><hl><br><footer><p>tomatenmhark.org und subdomains: <a href="/info/me">Mark</a>"#);
    if dbg {
        o.push_str(r#"<br><small><small>"#);
        let time_pagegen = Instant::now();
        o.push_str(&format!(
            "{}/{}ms querying + {:.2}ms pagegen",
            status
                .0
                .values()
                .filter_map(|v| v.4)
                .sum::<Duration>()
                .as_millis(),
            status.2.as_millis(),
            (time_pagegen - time_queried).as_micros() as f32 / 1000.0,
        ));
        o.push_str(r#"</small></small>"#);
    }
    o.push_str(r#"</p></footer>"#);
    o.push_str("</body></html>");
    RawHtml(o)
}

#[get("/int")]
async fn int() -> RawHtml<&'static str> {
    RawHtml(include_str!("int.html"))
}

#[rocket::launch]
fn rocket() -> _ {
    let data = Data::new_sync();
    rocket::build()
        .manage(data)
        .mount("/", routes![index, index_dbg, int])
        .mount(
            "/info",
            FileServer::new(
                "/srv/tomatenmhark-slashinfo/",
                Options::Index | Options::NormalizeDirs | Options::Missing,
            ),
        )
        .mount("/", Redirecter)
}
